export const Second = 1000;
export const Minute: number;
export const Quarter: number;
export const Hour: number;
export const Day: number;
export const Week: number;
export const Year: number;
export const Month: number;
export const Season: number;
export const ISODatePattern: RegExp;
export type TimeData = number | string | Date;
export function diffTime(end: TimeData, start?: TimeData, map?: Map<string, number>): {
    distance: number;
    unit: string;
};
export function formatDate(time?: TimeData, template?: string): string;
export function changeMonth(date: TimeData, delta: number): Date;
export type Constructor<T> = new (...args: any[]) => T;
export type TypeKeys<T, D> = {
    [K in keyof T]: T[K] extends D ? K : never;
}[keyof T];
export type PickData<T> = Omit<T, TypeKeys<T, Function>>;
export type DataKeys<T> = Exclude<keyof T, TypeKeys<T, Function>>;
export function isEmpty(value: any): boolean;
export function byteLength(raw: string): number;
export function uniqueID(): string;
export function differ<T>(target: Record<string, T>, source: Record<string, T>): Record<string, T>;
export type ResultArray<T> = T extends ArrayLike<infer D> ? D[] : T[];
export function likeArray(data: any): data is ArrayLike<any>;
export function makeArray<T>(data: T): ResultArray<T>;
export type IndexKey = number | string | symbol;
export type GroupKey<T extends Record<IndexKey, any>> = keyof T | IndexKey;
export type Iteratee<T extends Record<IndexKey, any>> = keyof T | ((item: T) => GroupKey<T> | GroupKey<T>[]);
export function groupBy<T extends Record<IndexKey, any>>(list: T[], iteratee: Iteratee<T>): Record<GroupKey<T>, T[]>;
export function parseJSON(raw: string): any;
export function parseTextTable<T = {}>(raw: string, header?: boolean, separator?: string): any[];
/**
 * CRC-32 algorithm forked from Bakasen's
 *
 * @see http://blog.csdn.net/bakasen/article/details/6043797
 */
export function makeCRC32(raw: string): string;
export type SHAAlgorithm = 'SHA-1' | 'SHA-256' | 'SHA-384' | 'SHA-512';
/**
 * @see https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/digest#Converting_a_digest_to_a_hex_string
 */
export function makeSHA(raw: string, algorithm?: SHAAlgorithm): Promise<string>;
export function sum(...data: number[]): number;
export function averageOf(...data: number[]): number;
export function varianceOf(data: number[], sample?: boolean): number;
export function standardDeviationOf(data: number[], sample?: boolean): number;
export function hypotenuseOf(...data: number[]): number;
export function carryFloat(raw: number, length: number): string;
export function fixFloat(raw: number, length?: number): string;
export function bootI18n<T extends Record<string, string>>(data: Record<string, T>, fallback?: string): {
    language: string;
    words: T;
};
export function isNumberLetter(raw?: string): boolean;
export function textJoin(...parts: string[]): string;
export function isXDomain(URI: string): boolean;
export type JSONValue = number | boolean | string | null;
export interface URLData<E = unknown> {
    [key: string]: JSONValue | JSONValue[] | URLData | URLData[] | E;
}
export function parseURLData(raw?: string): URLData;
export function buildURLData(map: string[][] | Record<string, any>): URLSearchParams;
/**
 * Blob logic forked from axes's
 *
 * @see http://www.cnblogs.com/axes/p/4603984.html
 */
export function blobFrom(URI: string): Blob;
export type SelfCloseTags = 'area' | 'base' | 'br' | 'col' | 'embed' | 'hr' | 'img' | 'input' | 'link' | 'meta' | 'param' | 'source' | 'track' | 'wbr';
export interface KeyboardEventHandlers {
    onKeyDown?: (event: KeyboardEvent) => any;
    onKeyPress?: (event: KeyboardEvent) => any;
    onKeyUp?: (event: KeyboardEvent) => any;
}
export interface MouseEventHandlers {
    onClick?: (event: MouseEvent) => any;
    onDblClick?: (event: MouseEvent) => any;
    onMouseDown?: (event: MouseEvent) => any;
    onMouseMove?: (event: MouseEvent) => any;
    onMouseUp?: (event: MouseEvent) => any;
    onMouseOver?: (event: MouseEvent) => any;
    onMouseOut?: (event: MouseEvent) => any;
    onWheel?: (event: WheelEvent) => any;
}
export interface PointerEventHandlers {
    onPointerDown?(event: PointerEvent): any;
    onPointerMove?(event: PointerEvent): any;
    onPointerUp?(event: PointerEvent): any;
    onPointerOver?(event: PointerEvent): any;
    onPointerOut?(event: PointerEvent): any;
    onPointerCancel?(event: PointerEvent): any;
    onPointerLockChange?(event: Event): any;
    onPointerLockError?(event: Event): any;
}
/**
 * Mouse & Pointer Events with no bubble
 */
export interface HoverEventHandlers {
    onMouseEnter?(event: MouseEvent): any;
    onMouseLeave?(event: MouseEvent): any;
    onPointerEnter?(event: PointerEvent): any;
    onPointerLeave?(event: PointerEvent): any;
    onGotPointerCapture?(event: PointerEvent): any;
    onLostPointerCapture?(event: PointerEvent): any;
}
export interface TouchEventHandlers {
    onTouchStart?: (event: TouchEvent) => any;
    onTouchEnter?: (event: TouchEvent) => any;
    onTouchMove?: (event: TouchEvent) => any;
    onTouchLeave?: (event: TouchEvent) => any;
    onTouchEnd?: (event: TouchEvent) => any;
    onTouchCancel?: (event: TouchEvent) => any;
}
/**
 * UI Events with no bubble
 */
export interface UIEventHandlers {
    onScroll?: (event: UIEvent) => any;
    onResize?: (Event: UIEvent) => any;
}
export interface AnimationEventHandlers {
    onTransitionStart?(event: TransitionEvent): any;
    onTransitionEnd?(event: TransitionEvent): any;
    onTransitionCancel?(event: TransitionEvent): any;
    onAnimationStart?(event: AnimationEvent): any;
    onAnimationEnd?(event: AnimationEvent): any;
    onAnimationCancel?(event: AnimationEvent): any;
}
/**
 * Events of every HTML Element itself
 */
export type BaseEventHandlers = KeyboardEventHandlers & MouseEventHandlers & HoverEventHandlers & TouchEventHandlers & UIEventHandlers & AnimationEventHandlers;
/**
 * Field Events with no bubble
 */
export interface FieldEventHandlers {
    onFocus?: (event: FocusEvent) => any;
    onBlur?: (event: FocusEvent) => any;
}
export interface InputEventHandlers {
    onFocusIn?(event: FocusEvent): any;
    onFocusOut?(event: FocusEvent): any;
    onCompositionStart?(event: CompositionEvent): any;
    onCompositionUpdate?(event: CompositionEvent): any;
    onCompositionEnd?(event: CompositionEvent): any;
    onInput?(event: InputEvent): any;
    onChange?(event: Event): any;
}
export interface FormEventHandlers {
    onSubmit?(event: Event): any;
    onReset?(event: Event): any;
}
/**
 * Events of every Container HTML Element
 */
export type BubbleEventHandlers = KeyboardEventHandlers & MouseEventHandlers & PointerEventHandlers & TouchEventHandlers & AnimationEventHandlers & InputEventHandlers & FormEventHandlers;
export type HTMLOwnKeys<T extends HTMLElement = HTMLElement> = Exclude<keyof T, keyof Node | keyof EventTarget>;
export type SVGOwnKeys<T extends SVGElement = SVGElement> = Exclude<keyof T, keyof Node | keyof EventTarget>;
export type HTMLContentKeys = 'innerHTML' | 'innerText' | 'textContent' | 'contentEditable';
export type CSSStyles = Partial<Omit<PickData<CSSStyleDeclaration>, 'length' | 'parentRule'> & Record<string, any>>;
export type DOMProps_Read2Write<T extends Partial<Element>> = {
    [K in keyof T]: T[K] extends HTMLElement ? string : T[K] extends DOMTokenList ? string : T[K] extends CSSStyleDeclaration ? CSSStyles : T[K];
};
export type BaseHTMLProps<T extends HTMLElement> = Partial<DOMProps_Read2Write<Pick<T, Extract<DataKeys<T>, HTMLOwnKeys<T>>>>>;
export type SVGProps_Read2Write<T extends Partial<SVGElement>> = {
    [K in keyof T]?: T[K] extends SVGAnimatedLength ? string : T[K] extends SVGAnimatedLengthList ? string : T[K] extends SVGAnimatedRect ? string : T[K] extends SVGAnimatedPreserveAspectRatio ? string : T[K];
};
export type BaseSVGProps<T extends SVGElement> = Partial<SVGProps_Read2Write<DOMProps_Read2Write<Pick<T, Extract<DataKeys<T>, SVGOwnKeys<T>>>>>>;
export interface HTMLProps extends BaseEventHandlers, Omit<BaseHTMLProps<HTMLElement>, HTMLContentKeys> {
    role?: string;
}
export interface SVGProps extends BaseEventHandlers, BaseSVGProps<SVGElement> {
}
export type HTMLContainerProps = BubbleEventHandlers & HTMLProps & Partial<Pick<HTMLElement, HTMLContentKeys>>;
export interface HTMLHyperLinkProps extends HTMLContainerProps {
    href?: string;
    target?: '_self' | '_parent' | '_top' | '_blank';
}
export type HTMLTableCellProps = HTMLContainerProps & Partial<Pick<HTMLTableCellElement, 'colSpan' | 'rowSpan'>>;
export type BaseFieldProps = Partial<Pick<HTMLInputElement, 'name' | 'defaultValue' | 'value' | 'required' | 'disabled'>>;
export interface BaseInputProps extends Partial<Pick<HTMLInputElement, 'readOnly' | 'placeholder'>> {
    list?: string;
}
export type TextFieldProps = BaseInputProps & Partial<Pick<HTMLInputElement, 'size' | 'minLength' | 'maxLength' | 'pattern' | 'autocomplete' | 'spellcheck'>>;
export type NumberFieldProps = BaseInputProps & Partial<Pick<HTMLInputElement, 'min' | 'max' | 'step'>>;
export type HTMLFieldInternals = Pick<HTMLInputElement, 'form' | 'validity' | 'validationMessage' | 'willValidate' | 'checkValidity' | 'reportValidity'>;
export interface HTMLFieldProps extends HTMLProps, BaseFieldProps, FieldEventHandlers, InputEventHandlers {
    autofocus?: HTMLInputElement['autofocus'];
}
export interface HTMLButtonProps extends HTMLFieldProps, HTMLContainerProps {
    type?: 'button' | 'image' | 'submit' | 'reset';
}
export interface HTMLInputProps extends HTMLFieldProps, BaseInputProps {
    type?: 'checkbox' | 'color' | 'date' | 'datetime-local' | 'email' | 'file' | 'hidden' | 'month' | 'number' | 'password' | 'radio' | 'range' | 'search' | 'tel' | 'text' | 'time' | 'url' | 'week' | HTMLButtonProps['type'];
}
export type HTMLField = HTMLInputElement & HTMLTextAreaElement & HTMLSelectElement & HTMLFieldSetElement;
/**
 * @see https://developers.google.com/web/fundamentals/web-components/customelements#reactions
 */
export interface CustomElement extends HTMLElement {
    /**
     * Called every time the element is inserted into the DOM
     */
    connectedCallback?(): void;
    /**
     * Called every time the element is removed from the DOM.
     */
    disconnectedCallback?(): void;
    /**
     * Called when an observed attribute has been added, removed, updated, or replaced.
     * Also called for initial values when an element is created by the parser, or upgraded.
     *
     * Note: only attributes listed in static `observedAttributes` property will receive this callback.
     */
    attributeChangedCallback?(name: string, oldValue: string, newValue: string): void;
    /**
     * The custom element has been moved into a new document
     * (e.g. someone called `document.adoptNode(el)`).
     */
    adoptedCallback?(): void;
}
/**
 * @see https://web.dev/more-capable-form-controls/#lifecycle-callbacks
 */
export interface CustomFormElement extends CustomElement, BaseFieldProps, HTMLFieldInternals {
    /**
     * Called when the browser associates the element with a form element,
     * or disassociates the element from a form element.
     */
    formAssociatedCallback?(form: HTMLFormElement): void;
    /**
     * Called after the disabled state of the element changes,
     * either because the disabled attribute of this element was added or removed;
     * or because the disabled state changed on a `<fieldset>` that's an ancestor of this element.
     *
     * @param disabled This parameter represents the new disabled state of the element.
     */
    formDisabledCallback?(disabled: boolean): void;
    /**
     * Called after the form is reset.
     * The element should reset itself to some kind of default state.
     */
    formResetCallback?(): void;
    /**
     * Called in one of two circumstances:
     *   - When the browser restores the state of the element (for example, after a navigation, or when the browser restarts). The `mode` argument is `"restore"` in this case.
     *   - When the browser's input-assist features such as form autofilling sets a value. The `mode` argument is `"autocomplete"` in this case.
     *
     * @param state The type of this argument depends on how the `this.internals.setFormValue()` method was called.
     * @param mode
     */
    formStateRestoreCallback?(state: string | File | FormData, mode: 'restore' | 'autocomplete'): void;
}
export function parseDOM(HTML: string): ChildNode[];
export function walkDOM<T extends Node = Node>(root: Node, type?: Node['nodeType']): Generator<T>;
export function getVisibleText(root: Element): string;
/**
 * Split a DOM tree into Pages like PDF files
 *
 * @param pageHeight the default value is A4 paper's height
 * @param pageWidth the default value is A4 paper's width
 */
export function splitPages({ offsetWidth, children }: HTMLElement, pageHeight?: number, pageWidth?: number): Element[][];
export interface CSSOptions extends Pick<HTMLLinkElement, 'title' | 'media' | 'crossOrigin' | 'integrity'> {
    alternate?: boolean;
}
export function importCSS(URI: string, { alternate, ...options }?: CSSOptions): Promise<CSSStyleSheet>;
export function insertToCursor(...nodes: Node[]): void;
export function scrollTo(selector: string, root?: Element): void;
export interface ScrollEvent {
    target: HTMLHeadingElement;
    links: (HTMLAnchorElement | HTMLAreaElement)[];
}
export function watchScroll(box: HTMLElement, handler: (event: ScrollEvent) => any, depth?: number): {
    level: number;
    id: string;
    text: string;
}[];
export function watchVisible(root: Element, handler: (visible: boolean) => any): void;
export function formToJSON<T = URLData<File>>(form: HTMLFormElement | HTMLFieldSetElement): T;
export function sleep(seconds?: number): Promise<unknown>;
export function asyncLoop(executor: (...data: any[]) => any, seconds?: number): () => boolean;
export function promisify<T extends Event>(scope: string, element: Element): Promise<T>;
export type MessageGlobal = Window | Worker;
export function createMessageServer(handlers: Record<string, (data: any) => any | Promise<any>>): () => void;
export function createMessageClient(target: Window | Worker, origin?: string): (type: string, data?: any) => Promise<unknown>;
export function serviceWorkerUpdate(registration: ServiceWorkerRegistration): Promise<ServiceWorker>;
export interface CartesianCoordinate {
    x: number;
    y: number;
    z?: number;
}
export class PageVector {
    from: CartesianCoordinate;
    to: CartesianCoordinate;
    constructor(from: CartesianCoordinate, to: CartesianCoordinate);
    get length(): number;
    get direction(): "left" | "right" | "down" | "up" | "forward" | "backward";
}
export function getSwipeVector(from: CartesianCoordinate, to: CartesianCoordinate, threshold?: number): PageVector;
export interface AnimationEvents {
    transition: TransitionEvent;
    animation: AnimationEvent;
}
export type AnimationType = keyof AnimationEvents;
export function durationOf(type: AnimationType, element: HTMLElement): number;
export function watchMotion<T extends AnimationType>(type: T, element: HTMLElement): Promise<void | AnimationEvents[T]>;
export function transitIn(element: HTMLElement, className: string, display?: string): Promise<void | TransitionEvent>;
export function animateIn(element: HTMLElement, className: string, display?: string): Promise<void | AnimationEvent>;
export function transitOut(element: HTMLElement, className: string, remove?: boolean): Promise<void>;
export function animateOut(element: HTMLElement, className: string, remove?: boolean): Promise<void>;

//# sourceMappingURL=index.d.ts.map
